import base64
import json
import aiofiles
import aiohttp
from cryptography.hazmat.primitives.serialization import load_pem_private_key
from cryptography.hazmat.backends import default_backend
import time
import os
import logging
from urllib.parse import urlparse

from accqsure.exceptions import AccQsureException


class Token(object):
    def __init__(
        self,
        organization_id: str,
        access_token: str,
        expires_at: int,
        api_endpoint: str,
    ):
        self.organization_id = organization_id
        self.access_token = access_token
        self.expires_at = expires_at
        self.api_endpoint = api_endpoint

    def to_json(self) -> str:
        return json.dumps(self.__dict__, indent=2, ensure_ascii=False)

    @classmethod
    def from_json(cls, json_str: str):
        data = json.loads(json_str)
        return cls(**data)

    def __repr__(self) -> str:
        return self.to_json()


def base64_to_base64_url(data):
    return data.replace("=", "").replace("+", "-").replace("/", "_")


async def sign_jwt(alg, kid, aud, iss, sub, exp, payload, private_key_pem):
    header = {
        "alg": alg,
        "kid": kid,
        "typ": "JWT",
    }

    full_payload = {
        **payload,
        "iat": int(time.time()),
        "exp": exp,
        "iss": iss,
        "sub": sub,
        "aud": aud,
    }

    partial_token = f"{base64_to_base64_url(base64.b64encode(json.dumps(header).encode()).decode())}.{base64_to_base64_url(base64.b64encode(json.dumps(full_payload).encode()).decode())}"

    private_key = load_pem_private_key(
        private_key_pem.encode(), password=None, backend=default_backend()
    )

    if alg == "EdDSA":
        signature = private_key.sign(partial_token.encode())
    else:
        raise ValueError("Unsupported algorithm")

    signed_token = (
        f"{partial_token}.{base64_to_base64_url(base64.b64encode(signature).decode())}"
    )
    return signed_token


async def get_access_token(key):

    try:
        token = await sign_jwt(
            "EdDSA",
            key["key_id"],
            key["auth_uri"],
            key["client_id"],
            key["client_id"],
            int(time.time()) + 3600,
            {"organization_id": key["organization_id"]},
            key["private_key"],
        )
    except Exception as error:
        raise AccQsureException(f"Error signing client JWT {error}")

    async with aiohttp.ClientSession() as session:
        async with session.post(
            key["auth_uri"],
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            data={
                "grant_type": "client_credentials",
                "client_id": key["client_id"],
                "scope": "admin",
                "client_assertion_type": "urn:ietf:params:oauth:client-assertion-type:jwt-bearer",
                "client_assertion": token,
            },
        ) as resp:
            try:
                access_token = await resp.json()
            except:
                error = resp.text
                raise AccQsureException(f"Error fetching access token {error}")

    api_url = urlparse(key["auth_uri"])
    return dict(
        organization_id=key["organization_id"],
        access_token=access_token["access_token"],
        expires_at=access_token["expires_at"],
        api_endpoint=f"{api_url.scheme}://{api_url.netloc}",
    )


async def load_cached_token(token_file_path):
    if not os.path.exists(token_file_path):
        return None

    async with aiofiles.open(token_file_path, mode="r", encoding="utf8") as f:
        try:
            raw = await f.read()
            data = Token.from_json(raw)
            return data
        except json.JSONDecodeError:
            return None


async def save_token(token_file_path: str, token: Token):
    os.makedirs(os.path.dirname(token_file_path), exist_ok=True)
    async with aiofiles.open(token_file_path, "w") as f:
        await f.write(token.to_json())
    os.chmod(token_file_path, 0o600)


def is_token_valid(token: Token):
    if not token:
        logging.debug(f"Token absent")
        return False
    logging.debug(f"Token expires: {token.expires_at}")
    return (token.expires_at - 60) > time.time()


class Auth(object):
    def __init__(self, config_dir: str, credentials_file: str, **kwargs):
        self.token_file_path = f"{config_dir}/token.json"
        self.credentials_file = credentials_file
        self.token = None
        self.key = None

    async def get_new_token(self):
        if not self.key:
            try:
                async with aiofiles.open(
                    self.credentials_file, mode="r", encoding="utf8"
                ) as f:
                    data = await f.read()

                self.key = json.loads(data)
            except FileNotFoundError:
                raise AccQsureException(
                    f"AccQsure credentials file {self.credentials_file} not found"
                )
        token = await get_access_token(self.key)
        logging.debug(f"Token Response {token}")
        self.token = Token(**token)
        logging.debug(f"New Token {self.token}")
        await save_token(self.token_file_path, self.token)

    async def get_token(self):
        if is_token_valid(self.token):
            logging.debug(f"Token is valid")
            return self.token
        else:
            if not self.token:
                logging.debug(f"Checking cached token")
                token = await load_cached_token(self.token_file_path)
                if is_token_valid(token):
                    self.token = token
                else:
                    await self.get_new_token()
            else:
                await self.get_new_token()

        logging.debug(f"Token expires: {self.token.expires_at}")

        return self.token



import click
from accqsure.cli import cli, pass_config


@cli.group()
@pass_config
def document_type(config):
    """AccQsure document_type commands."""
    pass


@document_type.command()
@pass_config
def list(config):
    """List document types."""
    data = [
        ["ID", "NAME", "CODE"],
        [
            "-" * 80,
            "-" * 80,
            "-" * 80,
        ],
    ]

    documents = config.accqsure.run(config.accqsure.client.document_types.list())
    for doc in documents:
        data.append(
            [
                doc.id,
                doc.name,
                doc.code,
            ]
        )
    for row in data:
        click.echo(
            "{: >26.24} {: >40.38} {: >14.12} " "".format(*row),
            file=config.stdout,
        )



import json
import os
import aiohttp
import logging
import traceback
from importlib.metadata import version

from accqsure.auth import Auth
from accqsure.text import Text
from accqsure.document_types import DocumentTypes
from accqsure.documents import Documents
from accqsure.manifests import Manifests

from accqsure.exceptions import ApiError, AccQsureException


DEFAULT_CONFIG_DIR = "~/.accqsure"
DEFAULT_CREDENTIAL_FILE_NAME = "credentials.json"


class AccQsure(object):
    def __init__(self, **kwargs):
        self._version = version("accqsure")
        config_dir = kwargs.get("config_dir") or os.path.expanduser(
            os.environ.get("ACCQSURE_CONFIG_DIR") or DEFAULT_CONFIG_DIR
        )
        credentials_file = kwargs.get("credentials_file") or os.path.expanduser(
            os.environ.get("ACCQSURE_CREDENTIALS_FILE")
            or f"{config_dir}/{DEFAULT_CREDENTIAL_FILE_NAME}"
        )
        self.auth = Auth(
            config_dir=config_dir,
            credentials_file=credentials_file,
        )
        self.text = Text(self)
        self.document_types = DocumentTypes(self)
        self.documents = Documents(self)
        self.manifests = Manifests(self)

    @property
    def __version__(self) -> str:
        return self._version

    async def _query(self, path, method, params=None, data=None, headers=None):
        try:
            token = await self.auth.get_token()
        except AccQsureException as e:
            raise e
        except Exception:
            raise AccQsureException(
                f"Error getting authorization tokens.  Verify configured credentials. Error: {traceback.format_exc()}"
            )
        logging.debug(
            f"Call parameters - Path: {path}, Method: {method}, Params: {params}, Body: {data}, Headers: {headers}"
        )
        api_endpoint = token.api_endpoint
        headers = (
            {
                **headers,
                **{
                    "Authorization": f"Bearer {token.access_token}",
                    "User-Agent": f"python-sdk/{self._version}",
                },
            }
            if headers
            else {
                "Authorization": f"Bearer {token.access_token}",
                "User-Agent": f"python-sdk/{self._version}",
            }
        )
        if params:
            if not isinstance(params, dict):
                raise AccQsureException("Query parameters must be a valid dictionary")
            params = {
                k: (str(v).lower() if isinstance(v, bool) else v)
                for k, v in params.items()
            }  ## aiohttp doesn't support boolean

        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json"
        url = f"{api_endpoint}/v1{path}"

        logging.debug(
            f"Request - Url: {url}, Method: {method}, Params: {params}, Body: {data}, Headers: {headers}"
        )
        async with aiohttp.ClientSession() as session:
            async with session.request(
                method,
                url,
                data=json.dumps(data),
                headers=headers,
                params=params,
            ) as resp:
                if (resp.status // 100) in [4, 5]:
                    what = await resp.read()
                    content_type = resp.headers.get("content-type", "")
                    resp.close()
                    if content_type == "application/json":
                        raise ApiError(resp.status, json.loads(what.decode("utf8")))
                    else:
                        raise ApiError(resp.status, {"message": what.decode("utf8")})
                results = await resp.json()
                return results

    async def _query_stream(self, path, method, params=None, data=None, headers=None):
        try:
            token = await self.auth.get_token()
        except AccQsureException as e:
            raise e
        except Exception:
            raise AccQsureException(
                f"Error getting authorization tokens.  Verify configured credentials. Error: {traceback.format_exc()}"
            )
        logging.debug(
            f"Call parameters - Path: {path}, Method: {method}, Params: {params}, Body: {data}, Headers: {headers}"
        )
        api_endpoint = token.api_endpoint
        headers = (
            {
                **headers,
                **{
                    "Authorization": f"Bearer {token.access_token}",
                    "User-Agent": f"python-sdk/{self._version}",
                },
            }
            if headers
            else {
                "Authorization": f"Bearer {token.access_token}",
                "User-Agent": f"python-sdk/{self._version}",
            }
        )
        if params:
            if not isinstance(params, dict):
                raise AccQsureException("Query parameters must be a valid dictionary")
            params = {
                k: (str(v).lower() if isinstance(v, bool) else v)
                for k, v in params.items()
            }  ## aiohttp doesn't support boolean

        if "Content-Type" not in headers:
            headers["Content-Type"] = "application/json"
        url = f"{api_endpoint}/v1{path}"

        logging.debug(
            f"Request - Url: {url}, Method: {method}, Params: {params}, Body: {data}, Headers: {headers}"
        )
        answer = ""
        async with aiohttp.ClientSession() as session:
            async with session.request(
                method,
                url,
                data=json.dumps(data),
                headers=headers,
                params=params,
            ) as resp:
                if (resp.status // 100) in [4, 5]:
                    what = await resp.read()
                    content_type = resp.headers.get("content-type", "")
                    resp.close()
                    if content_type == "application/json":
                        raise ApiError(resp.status, json.loads(what.decode("utf8")))
                    else:
                        raise ApiError(resp.status, {"message": what.decode("utf8")})
                try:
                    async for line in resp.content:
                        if line and line.strip():
                            clean_line = (
                                line.decode("utf-8").removeprefix("data:").strip()
                            )
                            logging.debug(clean_line)
                            if clean_line == "[DONE]":
                                continue
                            try:
                                response = json.loads(clean_line)
                            except:
                                logging.error("bad line", clean_line)
                                continue

                            if response.get("generated_text"):
                                logging.debug("final response", response)
                                return response.get("generated_text")
                            elif response.get("choices")[0].get("finish_reason"):
                                continue
                            else:
                                content = (
                                    response.get("choices")[0]
                                    .get("delta")
                                    .get("content")
                                )
                                answer += content
                    return answer
                except Exception as e:
                    logging.exception(f"Error during generation response")
                    data = await response.text()
                    logging.error(f"Response error: {data}")
                    raise e



import json
import logging

from accqsure.exceptions import SpecificationError


class Manifests(object):
    def __init__(self, accqsure):
        self.accqsure = accqsure

    async def get(self, id, **kwargs):
        resp = await self.accqsure._query(f"/manifest/{id}", "GET", kwargs)
        return Manifest(self.accqsure, **resp)

    async def get_global(self, **kwargs):
        resp = await self.accqsure._query(f"/manifest/global", "GET", kwargs)
        return Manifest(self.accqsure, **resp)

    async def list(self, document_type_id, **kwargs):
        resp = await self.accqsure._query(
            f"/manifest", "GET", dict(document_type_id=document_type_id, **kwargs)
        )
        manifests = [Manifest(self.accqsure, **manifest) for manifest in resp]
        return manifests

    async def create(
        self,
        document_type_id,
        name,
        reference_document_id,
        **kwargs,
    ):

        data = dict(
            name=name,
            document_type_id=document_type_id,
            reference_document_id=reference_document_id,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info(f"Creating Manifest {name}")
        resp = await self.accqsure._query("/manifest", "POST", None, payload)
        manifest = Manifest(self.accqsure, **resp)
        logging.info(f"Created Manifest {name} with id {manifest.id}")

        return manifest

    async def remove(self, id, **kwargs):
        await self.accqsure._query(f"/manifest/{id}", "DELETE", dict(**kwargs))


class Manifest:
    def __init__(self, accqsure, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._document_type_id = self._entity.get("document_type_id")
        self._name = self._entity.get("name")
        self._global = self._entity.get("global")
        self._reference_document = self._entity.get("reference_document")

    @property
    def id(self) -> str:
        return self._id

    @property
    def document_type_id(self) -> str:
        return self._document_type_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def reference_document_id(self) -> str:
        return self._reference_document.get("entity_id")

    @property
    def reference_document_doc_id(self) -> str:
        return self._reference_document.get("doc_id")

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"Manifest( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):
        await self.accqsure._query(
            f"/manifest/{self._id}",
            "DELETE",
        )

    async def rename(self, name):
        resp = await self.accqsure._query(
            f"/manifest/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):
        resp = await self.accqsure._query(
            f"/manifest/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def get_reference_contents(self):
        if not self._content_id:
            raise SpecificationError("content_id", "Content not uploaded for document")
        resp = await self.accqsure._query(
            f"/document/{self.id}/asset/{self._content_id}",
            "GET",
        )
        return resp

    async def list_checks(self, **kwargs):
        resp = await self.accqsure._query(f"/manifest/{self.id}/check", "GET", kwargs)
        checks = [ManifestCheck(self.accqsure, self, **check) for check in resp]
        return checks

    async def create_check(self, name, section, prompt, **kwargs):
        data = dict(
            name=name,
            section=section,
            prompt=prompt,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info(f"Creating Manifest Check {name}")
        resp = await self.accqsure._query(
            f"/manifest/{self.id}/check", "POST", None, payload
        )
        check = ManifestCheck(self.accqsure, self, **resp)
        logging.info(f"Created Manifest Check {name} with id {check.id}")

        return check

    async def remove_check(self, check_id, **kwargs):
        await self.accqsure._query(
            f"/manifest/{self.id}/check/{check_id}", "DELETE", dict(**kwargs)
        )


class ManifestCheck:
    def __init__(self, accqsure, manifest, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._manifest = manifest
        self._id = self._entity.get("entity_id")
        self._section = self._entity.get("section")
        self._name = self._entity.get("name")
        self._prompt = self._entity.get("prompt")

    @property
    def id(self) -> str:
        return self._id

    @property
    def section(self) -> str:
        return self._section

    @property
    def name(self) -> str:
        return self._name

    @property
    def prompt(self) -> str:
        return self._prompt

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"ManifestCheck( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):
        await self.accqsure._query(
            f"/manifest/{self._manifest.id}/check/{self.id}",
            "DELETE",
        )

    async def update(self, **kwargs):
        resp = await self.accqsure._query(
            f"/manifest/{self._manifest.id}/check/{self.id}",
            "PUT",
            None,
            dict(**kwargs),
        )
        self.__init__(self.accqsure, self._manifest, **resp)
        return self

    async def refresh(self):
        resp = await self.accqsure._query(
            f"/manifest/{self._manifest.id}/check/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, self._manifest, **resp)
        return self

    async def run(self, doc_content):
        reference_doc_content = await self._manifest.get_reference_contents()
        resp = await self.accqsure._query_stream(
            f"/manifest/{self._manifest.id}/check/{self.id}/run",
            "POST",
            None,
            dict(doc_content=doc_content, reference_doc_content=reference_doc_content),
        )
        return resp



import asyncio
import click
import logging
from os import devnull
from sys import stderr, stdout


from accqsure.accqsure import AccQsure


class AccQsureRunner(object):
    def __init__(self):
        self._accqsure_client = None

    @property
    def client(self) -> AccQsure:
        if self._accqsure_client is None:
            try:
                self._accqsure_client = AccQsure()
            except Exception as err:
                raise click.UsageError(err)
        return self._accqsure_client

    async def _run(self, *tasks):
        return await asyncio.gather(*tasks)

    def run(self, *tasks):
        try:
            if len(tasks) == 1:
                return_value = asyncio.run(*tasks)
            else:
                return_value = asyncio.run(self._run(*tasks))
        except Exception as err:
            raise click.UsageError(err)
        return return_value


class Config(object):
    def __init__(self):
        self.stderr = stderr
        self.stdout = stdout
        self.accqsure = AccQsureRunner()


pass_config = click.make_pass_decorator(Config, ensure=True)


@click.group()
@click.version_option(package_name="accqsure", prog_name="AccQsure CLI and SDK")
@click.option(
    "--debug",
    "-d",
    is_flag=True,
    type=click.BOOL,
    default=False,
    help="Show debug output.",
)
@click.option(
    "--output-file",
    "-o",
    type=click.File("w"),
    default="-",
    help="Send output to file.",
)
@click.option(
    "--silent",
    "-s",
    is_flag=True,
    type=click.BOOL,
    default=False,
    help="Silence all output.",
)
@click.option(
    "--verbose",
    "-v",
    "verbosity",
    count=True,
    type=click.INT,
    default=0,
    help="Specify verbosity (repeat to increase).",
)
@pass_config
def cli(config, debug, output_file, silent, verbosity):
    """AccQsure command-line interface."""
    config.stdout = output_file

    if debug or verbosity > 0:
        if silent:
            click.echo(
                "Ignoring silent flag when debug or verbosity is set.",
                file=config.stderr,
            )
        if verbosity == 1:
            verbosity = logging.INFO
        else:
            verbosity = logging.DEBUG
    elif silent:
        config.stderr = config.stdout = open(devnull, "w")
    else:
        verbosity = logging.WARNING

    if verbosity != logging.WARNING:  # default
        click.echo(
            f"Verbosity set to {logging.getLevelName(verbosity)}",
            file=config.stderr,
        )

    logging.basicConfig(
        format="%(asctime)s.%(msecs)03dZ  %(levelname)s  %(message)s",
        datefmt="%Y-%m-%dT%H:%M:%S",
        level=verbosity,
        stream=config.stderr,
        force=True,
    )


from accqsure.cli.text import text
from accqsure.cli.document import document
from accqsure.cli.document_type import document_type
from accqsure.cli.manifest import manifest



import click
from accqsure.cli import cli, pass_config


@cli.group()
@pass_config
def document(config):
    """AccQsure document commands."""
    pass


@document.command()
@click.argument("document_type_id", type=click.STRING)
@pass_config
def list(config, document_type_id):
    """List documents."""
    data = [
        ["ID", "NAME", "DOC_ID"],
        [
            "-" * 80,
            "-" * 80,
            "-" * 80,
        ],
    ]

    documents = config.accqsure.run(
        config.accqsure.client.documents.list(document_type_id=document_type_id)
    )
    for doc in documents:
        data.append(
            [
                doc.id,
                doc.name,
                doc.doc_id,
            ]
        )
    for row in data:
        click.echo(
            "{: >26.24} {: >40.38} {: >14.12} " "".format(*row),
            file=config.stdout,
        )



class Text(object):
    def __init__(self, accqsure):
        self.accqsure = accqsure

    async def generate(self, messages, max_tokens=2048, temperature=0.8, **kwargs):
        resp = await self.accqsure._query_stream(
            f"/text/generate",
            "POST",
            None,
            {
                **kwargs,
                **dict(
                    messages=messages, max_tokens=max_tokens, temperature=temperature
                ),
            },
        )
        return resp

    async def vectorize(self, inputs, **kwargs):
        resp = await self.accqsure._query(
            f"/text/vectorize", "POST", None, {**kwargs, **dict(inputs=inputs)}
        )
        return resp

    async def tokenize(self, inputs, **kwargs):
        resp = await self.accqsure._query(
            f"/text/tokenize", "POST", None, {**kwargs, **dict(inputs=inputs)}
        )
        return resp



import json
import logging

from accqsure.exceptions import SpecificationError
from accqsure.manifests import Manifest


class Documents(object):
    def __init__(self, accqsure):
        self.accqsure = accqsure

    async def get(self, id, **kwargs):
        resp = await self.accqsure._query(f"/document/{id}", "GET", kwargs)
        return Document(self.accqsure, **resp)

    async def list(self, document_type_id, **kwargs):
        resp = await self.accqsure._query(
            f"/document", "GET", dict(document_type_id=document_type_id, **kwargs)
        )
        documents = [Document(self.accqsure, **document) for document in resp]
        return documents

    async def create(
        self,
        document_type_id,
        name,
        doc_id,
        **kwargs,
    ):

        data = dict(
            name=name,
            document_type_id=document_type_id,
            doc_id=doc_id,
            **kwargs,
        )
        payload = {k: v for k, v in data.items() if v is not None}
        logging.info(f"Creating Document {name}")
        resp = await self.accqsure._query("/document", "POST", None, payload)
        document = Document(self.accqsure, **resp)
        logging.info(f"Created Document {name} with id {document.id}")

        return document

    async def markdown_convert(self, title, type, base64_contents, **kwargs):
        resp = await self.accqsure._query(
            f"/document/convert",
            "POST",
            None,
            {**kwargs, **dict(title=title, type=type, base64_contents=base64_contents)},
        )
        return resp

    async def remove(self, id, **kwargs):
        await self.accqsure._query(f"/document/{id}", "DELETE", dict(**kwargs))


class Document:
    def __init__(self, accqsure, **kwargs):
        self.accqsure = accqsure
        self._entity = kwargs
        self._id = self._entity.get("entity_id")
        self._document_type_id = self._entity.get("document_type_id")
        self._name = self._entity.get("name")
        self._doc_id = self._entity.get("doc_id")
        self._content_id = self._entity.get("content_id")

    @property
    def id(self) -> str:
        return self._id

    @property
    def document_type_id(self) -> str:
        return self._document_type_id

    @property
    def name(self) -> str:
        return self._name

    @property
    def doc_id(self) -> str:
        return self._doc_id

    def __str__(self):
        return json.dumps({k: v for k, v in self._entity.items()})

    def __repr__(self):
        return f"Document( accqsure , **{self._entity.__repr__()})"

    def __bool__(self):
        return bool(self._id)

    async def remove(self):
        await self.accqsure._query(
            f"/document/{self._id}",
            "DELETE",
        )

    async def rename(self, name):
        resp = await self.accqsure._query(
            f"/document/{self._id}",
            "PUT",
            None,
            dict(name=name),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def refresh(self):
        resp = await self.accqsure._query(
            f"/document/{self.id}",
            "GET",
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def get_contents(self):
        if not self._content_id:
            raise SpecificationError("content_id", "Content not uploaded for document")
        resp = await self.accqsure._query(
            f"/document/{self.id}/asset/{self._content_id}",
            "GET",
        )
        return resp

    async def set_contents(self, file_name, contents):
        resp = await self.accqsure._query(
            f"/document/{self.id}/asset/",
            "POST",
            dict(file_name=file_name),
            contents,
            {
                "Content-Type": "text/plain",
            },
        )
        asset_id = resp.get("asset_id")
        resp = await self.accqsure._query(
            f"/document/{self._id}",
            "PUT",
            None,
            dict(content_id=asset_id),
        )
        self.__init__(self.accqsure, **resp)
        return self

    async def list_manifests(self):
        resp = await self.accqsure._query(
            f"/document/{self.id}/manifest",
            "GET",
        )
        manifests = [Manifest(self.accqsure, **manifest) for manifest in resp]
        return manifests



import logging
from .accqsure import AccQsure

logging.basicConfig(
    format="%(asctime)s.%(msecs)03dZ  %(levelname)s  %(message)s",
    datefmt="%Y-%m-%dT%H:%M:%S",
)
logger = logging.getLogger(__name__)


__version__ = "0.1.5"
__all__ = "AccQsure"
